moude_dic = {1: "MathWork", 2: "showData", 3: "Uncertainty"}


def init():
    print("+" + "-" * 40 + "+")


def print_function(str, function_name, moude_id):
    print("|", end="")
    print(str)
    print("|", end="")
    print(function_name)
    print("|", end="")
    print("所属模块:\t" + moude_dic[moude_id])
    end()


def end():
    print("+" + "-" * 40 + "+")


def showFunction():
    init()
    print_function("生产均差表:", "AveDevCalTab(xlist, ylist)", 1)
    print_function("生产差分表:", "DiffTable(xlist, ylist)", 1)
    print_function("牛顿差值多项式求值:", "NewtonChaZhi(xlist, ylist, x_in)", 1)
    print_function("牛顿前插公式求值:", "NewtonQianCha(xlist, ylist, x_in)", 1)
    print_function("绘制数据曲线:", "printData(x_list,y_list)", 2)
    print_function("绘制多张数据曲线:", "printNpic()", 2)
    print_function("计算不确定度:", "CalUncer(data_list)", 3)



from decimal import Decimal


def CalUncer(data_list):
    data_num = len(data_list)
    data_sum = sum(data_list)
    data_Average = Decimal(str(data_sum)) / Decimal(str(data_num))
    L2_sum = Decimal('0')
    for data in data_list:
        L2_sum += (Decimal(str(data)) - data_Average) ** Decimal('2')
    Ua = (L2_sum / (data_num * data_num - Decimal('1'))) ** Decimal('0.5')
    E = Ua / data_Average
    print("+--------------------------------------+")
    print("|  平均值是:     \t{:^12.6f}       |".format(data_Average))
    print("|  A类不确定度是: \t{:^12.6f}       |".format(Ua))
    print("|  相对不确定度是: \t{:^12.6f}%      |".format(E * Decimal('100')))
    print("+--------------------------------------+")



# import matplotlib.pyplot as plt
try:
    import matplotlib.pyplot as plt
except ImportError:
    import os
    os.system('pip install matplotlib')
    import matplotlib.pyplot as plt


def printData(x_list,y_list):
    print("+-------------------------------+")
    x_label=input("|  请输入x轴英文名称:\t")
    y_label=input("|  请输入y轴英文名称:\t")
    title=  input("|  标题的英文是:     \t")
    print("+-------------------------------+")
    plt.plot(x_list,y_list)
    plt.xlabel(x_label)
    plt.ylabel(y_label)
    plt.title(title)
    plt.grid()
    plt.show()

def printNpic():
    print("+-------------------------------+")
    pic_row=int(input("|  请输入行数:\t"))
    pic_col=int(input("|  请输入列数:\t"))
    print("+-------------------------------+")
    print("")
    print("+-------------------------------+")
    for index in range(pic_col*pic_row):
        print("+  第"+str(index+1)+"张图:")
        plt.subplot(pic_row,pic_col,index+1)
        x_list=eval(input("|  请输入x坐标的数据列表[]:"))
        y_list=eval(input("|  请输入y坐标的数据列表[]:"))
        x_label = input("|  请输入x轴英文名称:\t")
        y_label = input("|  请输入y轴英文名称:\t")
        title = input("|  标题的英文是:     \t")
        plt.plot(x_list, y_list)
        plt.xlabel(x_label)
        plt.ylabel(y_label)
        plt.title(title)
        plt.grid()
    print("+-------------------------------+")
    plt.show()





from decimal import Decimal


def AveDevCalTab(xlist, ylist):
    x_list = []
    y_list = []
    for x in xlist:
        x_list.append(Decimal(str(x)))
    for y in ylist:
        y_list.append(Decimal(str(y)))
    DEV = []
    DEV.append(y_list)
    N = len(x_list)
    for jie in range(1, N):
        n = len(DEV[jie - 1])
        dev = []
        for i in range(n):
            if (i < jie):
                dev.append(Decimal('0'))
            else:
                ave_dev = (DEV[jie - 1][i] - DEV[jie - 1][i - 1]) / (x_list[i] - x_list[i - jie])
                dev.append(ave_dev)
        DEV.append(dev)
    print("均差计算表如下:")
    print("|{:^12}|{:^12}".format("X", "Y"), end="|")
    for i in range(1, N):
        print("{:^12}|".format("N" + str(i)), end="")
    print("")
    for i in range(N):
        print("|", end="")
        print("{:^12.5f}".format(x_list[i]), end="|")
        for j in range(N):
            if (i < j):
                print("{:^12}".format(""), end="|")
            else:
                print("{:^12.5f}".format(DEV[j][i]), end="|")
        print("")


def NewtonChaZhi(xlist, ylist, x_in):
    x_list = []
    y_list = []
    x = Decimal(str(x_in))
    for x in xlist:
        x_list.append(Decimal(str(x)))
    for y in ylist:
        y_list.append(Decimal(str(y)))
    DEV = []
    DEV.append(y_list)
    N = len(x_list)
    for jie in range(1, N):
        n = len(DEV[jie - 1])
        dev = []
        for i in range(n):
            if (i < jie):
                dev.append(Decimal('0'))
            else:
                ave_dev = (DEV[jie - 1][i] - DEV[jie - 1][i - 1]) / (x_list[i] - x_list[i - jie])
                dev.append(ave_dev)
        DEV.append(dev)
    y = Decimal('0')
    for i in range(N):
        a = DEV[i][i]
        p = Decimal('1')
        for j in range(i):
            p *= (Decimal(str(x)) - x_list[j])
        y += a * p
    return y


def DiffTable(xlist, ylist):
    x_list = []
    y_list = []
    for x0 in xlist:
        x_list.append(Decimal(str(x0)))
    for y0 in ylist:
        y_list.append(Decimal(str(y0)))
    h = x_list[1] - x_list[0]
    flag = 1
    for i in range(len(x_list) - 1):
        if (x_list[i + 1] - x_list[i] - h != 0):
            flag = 0
            break
    if (flag == 0):
        print("输入的x不是等差的")
        return -1
    else:
        dif_mat = []
        dif_mat.append(y_list)
        N = len(x_list)
        for jie in range(1, N):
            n = len(dif_mat[jie - 1])
            dev = []
            for i in range(n):
                if (i < jie):
                    dev.append(0)
                else:
                    ave_dev = (dif_mat[jie - 1][i] - dif_mat[jie - 1][i - 1])
                    dev.append(ave_dev)
            dif_mat.append(dev)
        print("差分计算表如下:")
        print("|{:^12}|{:^12}".format("X", "Y"), end="|")
        for i in range(1, N):
            print("{:^12}|".format("f^" + str(i)), end="")
        print("")
        for i in range(N):
            print("|", end="")
            print("{:^12.5f}".format(x_list[i]), end="|")
            for j in range(N):
                if (i < j):
                    print("{:^12}".format(""), end="|")
                else:
                    print("{:^12.5f}".format(dif_mat[j][i]), end="|")
            print("")


def NewtonQianCha(xlist, ylist, x_in):
    x_list = []
    y_list = []
    x = Decimal(str(x_in))
    for x0 in xlist:
        x_list.append(Decimal(str(x0)))
    for y0 in ylist:
        y_list.append(Decimal(str(y0)))
    h = x_list[1] - x_list[0]
    flag = 1
    for i in range(len(x_list) - 1):
        if (x_list[i + 1] - x_list[i] - h != 0):
            flag = 0
            break
    if (flag == 0):
        print("输入的x不是等差的")
        return -1
    else:
        dif_mat = []
        dif_mat.append(y_list)
        N = len(x_list)
        for jie in range(1, N):
            n = len(dif_mat[jie - 1])
            dev = []
            for i in range(n):
                if (i < jie):
                    dev.append(0)
                else:
                    ave_dev = (dif_mat[jie - 1][i] - dif_mat[jie - 1][i - 1])
                    dev.append(ave_dev)
            dif_mat.append(dev)
        y = dif_mat[0][0]
        t = (Decimal(str(x)) - x_list[0]) / h
        for jieshu in range(1, N):
            a = dif_mat[jieshu][jieshu]
            p = Decimal('1')
            jiecheng_shu = Decimal('1')
            for x in range(jieshu):
                jiecheng_shu *= Decimal(str(x + 1))
            for j in range(jieshu):
                p *= (t - Decimal(str(j)))
            y += a * p / jiecheng_shu
        return y



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ZxSci",
    version="0.2.3",
    author="Larry",
    author_email="1140091006@qq.com",
    description="一个帮助理工科本科学生进行简化计算的库",
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/sampleproject",
    license="MIT Licence",
    # packages=setuptools.find_packages("自制库/ZxSci"),
    # packages=["matplotlib"],
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    # include_package_data = True,
    # platforms = "any",
    # install_requires = ['chardet']

)